# -*- coding: utf-8 -*-
from .base import Response

class LastedResponse(Response):
	def __init__(self,title=None, url=None, release=None):
		super().__init__()
		self.title = title
		self.url = url
		self.release = release

	def read(self, obj):
		self.title = obj.get("title", None)
		self.url = obj.get("title", None)
		self.release = obj.get("release", None)

class LastedOp(Response):
	def __init__(self, result=None):
		super().__init__()
		self.result = result

	def read(self, obj):
		for k in obj.get("result", []):
			if self.result is None:
				self.result = []
			r = LastedResponse()
			r.read(k)
			self.result.append(r)

class SearchResponse(Response):
	def __init__(self, title=None, url=None, release=None):
		super().__init__()
		self.title = title
		self.url = url
		self.release =release

	def read(self, obj):
		self.title = obj.get("title", None)
		self.url = obj.get("url", None)
		self.release = obj.get("release", None)

class SearchOp(Response):
	def __init__(self, results=None, amount=None):
		super().__init__()
		self.results = results
		self.amount = amount

	def read(self, obj):
		self.amount = obj.get("amount", None)
		for k in obj.get("results", []):
			if self.results is None:
				self.results = []
			r = SearchResponse()
			r.read(k)
			self.results.append(r)

class StreamDownload(Response):
	def __init__(self,
						google=None,
						nofile=None,
						uptobox=None,
						zippyshare=None,
						megaup=None,
						elsfile=None,
						download=None):
		super().__init__()
		self.google = google
		self.nofile = nofile
		self.zippyshare = zippyshare
		self.megaup = megaup
		self.download = download
		self.elsfile = elsfile

	def read(self, obj):
		for i in obj:
			for k in obj[i]:
				self.google = k.get("google_drive", None)
				self.elsfile = k.get("elsfile", None)
				self.zippyshare = k.get("zippyshare", None)
				self.megaup = k.get("megaup", None)
				self.nofile = k.get("nofile", None)
				self.download = k.get("download", None)

class StreamInfo(Response):
	def __init__(self,
					sinopsis=None,
					thumb=None,
					status=None,
					tipe=None,
					durasi=None,
					genre=None):
		super().__init__()
		self.sinopsis = sinopsis
		self.thumb = thumb
		self.status = status
		self.tipe = tipe
		self.durasi = durasi
		self.genre = genre

	def read(self, obj):
		self.sinopsis = obj.get("sinopsis", None)
		self.thumb = obj.get("thumb", None)
		self.status = obj.get("status", None)
		self.tipe = obj.get("tipe", None)
		self.durasi = obj.get("durasi", None)
		self.genre = obj.get("genre", None)

class StreamOp(Response):
	def __init__(self, stream=None, info=None, list_downloads=None):
		super().__init__()
		self.stream = stream
		self.info = info
		self.list_downloads = list_downloads

	def read(self, obj):
		self.stream = obj.get("stream", None)
		info = StreamInfo()
		info.read(obj.get("info",None))
		self.info = info
		for k in obj.get("list_downloads",[]):
			if self.list_downloads is None:
				self.list_downloads = []
			r = StreamDownload()
			r.read(k)
			self.list_downloads.append(r)



from . import *

class OpzApi(SearchOpLoverz, LastedOpLoverz, StreamOpLoverz, Request):
	def __init__(self, return_as = "dict"):
		self.return_as = return_as
		super().__init__()

	def search_anime(self, query: str, page=1):
		data = self.detailed(query, page)
		if self.return_as == "dict":
			return data
		if self.return_as == "rpc":
			r = SearchOp()
			r.read(data)
			return r
		else:
			raise Exception("Makse sure your add right value for returning")

	def detail_anime(self, urls:list or str = None):
		data = self.get_stream(urls)
		if self.return_as == "dict":
			return data
		if self.return_as == "rpc":
			r = StreamOp()
			r.read(data)
			return r
		else:
			raise Exception("Makse sure your add right value for returning")

	def multi_detail(self, query: str, page: int = 1):
		#this may make your result getting slow
		#this will returned all data at one page
		#TODO: just using dict result for this, make you easy to parse this
		self.find_url_stream(query, page)
		for i in self.multi_stream():
			return i

	def lasted_anime(self, page: int = 1):
		data = self.lasted(page)
		if self.return_as == "dict":
			return data
		if self.return_as == "rpc":
			r = LastedOp()
			r.read(data)
			return r
		else:
			raise Exception("Makse sure your add right value for returning")



# -*- coding: utf-8 -*-
#JSON-RPC

class Response(object):
	def __init__(self):
		super(Response, self).__init__()

	def read(self):
		pass

	def write(self):
		pass

	def __getitem__(self, key):
		return getattr(self, key, None)

	def __setitem__(self, key, val):
		return setattr(self, key, val)

	def keys(self):
		return self.__dict__.keys()

	def values(self):
		return self.__dict__.values()

	def items(self):
		return self.__dict__.items()

	def __eq__(self, obj):
		return self.__class__.__name__ == obj.__class__.__name__

	def __repr__(self, indent=4):
		L=['%s=%r'%(key, value) for key, value in self.__dict__.items()]
		return '%s(%s)'%(self.__class__.__name__, ', '.join(L))


from .base import Response
from .style import LastedOp, LastedResponse, SearchOp, SearchResponse,StreamInfo,StreamOp,StreamDownload
from .util import LastedOpLoverz, SearchOpLoverz, StreamOpLoverz, Request
from .api import OpzApi
__author__ = "Dyseo"
__version__ ="0.0.1"
__all__ = [
	"OpzApi",
	"Response",
	"LastedResponse",
	"LastedOp",
	"SearchOp",
	"SearchResponse",
	"StreamOp",
	"StreamInfo",
	"StreamDownload",
	"LastedOpLoverz",
	"SearchOpLoverz",
	"StreamOpLoverz",
	"Request"
	]



# -*- coding: utf-8 -*-
import requests
from bs4 import BeautifulSoup
from itertools import zip_longest, chain, repeat
from concurrent.futures import ThreadPoolExecutor

URL = "https://www.oploverz.in"
header = {
		"User-Agent":"python-requests/2.21.0",
	}

class Request(object):
	def __init__(self, headers: set or dict = None, proxy: set or dict = None):
		self._req = requests.Session()
		self.headers = headers
		if self.headers is None:
			self._req.headers.update(header)
		self.proxy = proxy

	def _request(self, search=None, page=1):
		with self._req as s:
			if search != None:
				r = s.get(f"{URL}/page/{page}/?s={search}&post_type=post", proxies=self.proxy if self.proxy is not None else None)
			else:
				r = s.get(f"{URL}/page/{page}/", proxies = self.proxy if self.proxy != None else None)
			if r.ok:
				soup = BeautifulSoup(r.text, "lxml")
				body = soup.find("div", class_="right").find("div", class_="lts")
				selected = body.select("ul > li")
				return selected

class LastedOpLoverz(Request):
	def __init__(self):
		super(LastedOpLoverz, self).__init__()

	def lasted(self, page):
		ret = self._request(page)
		data = {"result":[]}
		for i in ret:
			prebody = i.find("div", class_="dtl")
			href = prebody.find("h2").find("a")
			rilis = prebody.find_all("span")[2].text
			data["result"].append({
				"title":href.text,
				"url":href["href"],
				"release":rilis.strip()
			})
		return data

class StreamOpLoverz(Request):
	def __init__(self, workers=50):
		self.workers = workers
		self.urls = []
		super(StreamOpLoverz, self).__init__()

	def find_url_stream(self, query, page):
		data = self._request(query, page)
		for p in data:
			ret = p.find("div", class_="dtl")
			url = ret.find("h2").find("a")["href"]
			self.urls.append(url)
		return self.urls

	def multi_stream(self):
		with ThreadPoolExecutor(max_workers=self.workers) as pool:
			lists = list(pool.map(self.get_stream, self.urls))
			return lists

	def this_info(self, data):
		ret = {}
		info = data.find("div", class_="animeinfo")
		thumb = info.find("img")["src"]
		detail = info.find_all("span")[:4]
		sinopsis = data.find("div", class_="sinop").text
		for i in detail:
			t , h= i.text.split(":")
			ret.update({
				"sinopis":sinopsis,
				"thumb":thumb,
				t.lower():h
			})
		return ret

	def get_stream(self, list_url: str or list):
		data = {"stream":"", "info":{},"list_downloads":[]}
		r = self._req.get(list_url)
		if r.ok:
			soup = BeautifulSoup(r.text, "lxml")
			post_body = soup.find("div", class_="postbody")
			stream_body = post_body.find("div", attrs={"id":"embed"})
			stream = stream_body.find("div", class_="embedholder").find("iframe").get("src")
			dwl = post_body.find("div", class_="soraddl op-download")
			info = self.this_info(post_body)
			ex = ["480p","720p","720p_10bit", "1080p"]
			for item, ext in zip_longest(dwl.find_all("div", class_="soraurl list-download"), ex):		
				href = item.find_all("a")
				title = [i.text.lower().replace(" ","_") for i in href]
				url = [i["href"] for i in href]
				data["stream"] = stream
				data["info"] = info
				#make it simple, Reff https://stackoverflow.com/a/23030666
				data["list_downloads"].append({
					ext.strip():[
					dict(zip(title, chain(url, repeat(None))))
					]
				})
			return data

class SearchOpLoverz(Request):
	def __init__(self):
		super(SearchOpLoverz, self).__init__()

	def detailed(self, search, page):
		select = self._request(search, page)
		data = {"amount":len(select),"results":[]}
		for i in select:
			prebody = i.find("div", class_="dtl")
			href = prebody.find("h2").find("a")
			rilis = prebody.find_all("span")[2].text
			data["results"].append({
				"title":href.text,
				"url":href["href"],
				"release":rilis.strip()
				})
		return data



from setuptools import setup, find_packages

with open("README.md", "r") as fp:
	long_desrib = fp.read()
	
install_require = [
					"requests",
					"bs4",
					"lxml"]

#package = ["pyoploverz"]

setup(
	name="pyoploverz",
	version="0.0.1",
	description="Unofficial wrapper anime from oploverz.in",
	author="Dyseo",
	author_email="katro.coplax@gmail.com",
	long_description=long_desrib,
	long_description_content_type="text/markdown",
	url="https://github.com/dyseo/PyOpLoverz",
	packages=find_packages(),
	classifiers=[
		"Programming Language :: Python :: 3.6",
		"Programming Language :: Python :: 3.7",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
		"Topic :: Software Development :: Libraries :: Python Modules",
		"Topic :: Utilities",
		"Topic :: Internet",
	]
)
